
//--------------------------------------------------------------------------
/*	Here we introduce an inner class called Particle. The Dendron applet 
 		will use an array of several tens of thousands of these Particles 
 		to simulate Diffusion-Limited Aggregation. Each Particle is defined by 
 		a two-dimensional position and velocity, and offers a method by which
 		the particle can be reset() or updated. */


class Particle {
  final float A = 0.9f;
  final float B = 1.0f - A;

  float 	px, py;		// Particle position
  float 	vx, vy;		// Particle velocity
  float 	dx, dy;		// Handy offsets
  int 	  used;
  boolean active;

  public Particle(){ 
    reset(); 
    reset2();
  }

  void reset(){
    // Reset the Particle to a new random location
    // and a zeroed-out velocity.
    px = (float) (Math.random() * (float) rasterWidth);
    py = (float) (Math.random() * (float) rasterHeight);
    vx = 0;
    vy = 0;
  }

  void reset2(){
    active = true;
    used = 0;
  }


  void update(){
    // This method advances a Particle to its next
    // location, based on its current velocity, and
    // also evolves the Particle's velocity somewhat
    // in order to produce a smoothed-out drunk walk.

    // Coarsely integrate the current velocity
    // in order to update the Particle's position.
    px += vx; 
    py += vy;

    // Do a bounds check to make sure that the Particle
    // stays within the bounds of the raster area.
    if (px > rasterWidth)  { 
      px -= rasterWidth; 
    }
    else if (px < 0     )  { 
      px += rasterWidth; 
    }
    if (py > rasterHeight) { 
      py -= rasterHeight; 
    }
    else if (py < 0      ) { 
      py += rasterHeight; 
    }

    // Compute a random deflection for the Particle,
    // and modify its velocity slightly in that direction.
    dx = (float)(Math.random()-0.5)*maxParticleVelocity;
    dy = (float)(Math.random()-0.5)*maxParticleVelocity;
    vx = A*vx + B*dx; 
    vy = A*vy + B*dy; 

  }

}
