// Dendron by Golan Levin
// http://www.flong.com/projects/dendron
// Java 1.1 version, October 2001
// Processing-135 version, January 2008


//---------------------------------------------------------------------------
color   myPalette[];
int 	rasterWidth;			// The width of the applet's pixel field.
int 	rasterHeight;			// The height of the applet's pixel field.
byte 	rasterPixels[];			// The array of bytes containing the pixels.
int 	numberOfRasterPixels;	        // The size (length) of this byte array.
final int UNSIGN = 0xFF;		

//----------------------------------------------------------------------------
/* 	Declare the variables and objects we'll be using in the Diffusion-Limited
 		Aggregation (DLA) simulation. Our simulation will consist of a set of 
 		Particles which move around the raster 
*/

int 		numberOfParticles;
Particle 	particleArray[];
float           aggregationThreshold = 10;
float           maxParticleVelocity  = 20;	
byte            DRAW_BYTE = (byte)129;//97;	
byte            ACCRETE_BYTE = (byte)49;//65;	
int             blurFrequency = 4;			

PFont F;

//----------------------------------------------------------------------------
void setup(){
  size (640, 480); 
  F = loadFont("BodoniMT-Italic-16.vlw");

  rasterWidth = width;
  rasterHeight = height;
  numberOfRasterPixels = rasterWidth * rasterHeight;
  rasterPixels = new byte[numberOfRasterPixels];
  for (int i=0; i<numberOfRasterPixels; i++){ 
    rasterPixels[i] = (byte) (0); 
  }

  myPalette = new color[256];
  createCustomPalette();
  initializeSimulation();
}

//--------------------------------------------------------------------------
void draw(){
  computeSimulation();
  loadPixels();
  
  int row;
  int index = 0;
  color c;
  for (int y=0; y<rasterHeight; y++){
    row = y*rasterWidth;
    for (int x=0; x<rasterWidth; x++){
      index = row+x;
      pixels[index] = myPalette[UNSIGN & rasterPixels[index]];
    }
  }
  updatePixels();
  
  drawText();
  
  
}


boolean clicked = false;
long clickedMillis = 0;
void drawText(){
  long now = millis();
  long elapsed = now - clickedMillis;
  
  float alp = 0.0;
  if (clicked == false){
    alp = 255.0;
  }
  if ((clicked == true) && (elapsed < 1500)){
      float frac = elapsed / 1500.0;
      frac = pow(frac, 0.5);
      frac = min(1.0, frac);
    alp = 255.0 * (1.0 - frac);
  }
  
  if (alp > 0){
  fill(176,245,218, alp);
  textAlign(CENTER);
  textFont(F);
  text("dendron by golan levin (2001, 2008)\nhttp://www.flong.com\nclick, drag, wait\n", width*0.5, height*0.4); 
  }
}



//--------------------------------------------------------------------------
void mousePressed(){
  aggregationThreshold = 1;
  createLine  (mouseX-1, mouseY, mouseX, mouseY);
  if (clicked == false){
     clickedMillis = millis();
  }
  clicked = true;
}

//--------------------------------------------------------------------------
void mouseDragged (){
  createLine  (pmouseX, pmouseY, mouseX, mouseY);
}

//--------------------------------------------------------------------------
void createCustomPalette(){
  // Create the palette we'll use to display the pixels.
  // This 256-color palette blends between the background and foreground
  // colors that were specified in the HTML parameter tags. We assume that
  // readAppletParameters() has already been called, which reads these
  // parameters into the myBackgroundColor and myForegroundColor objects.

  // Fetch the individual color components
  color myBackgroundColor = color(50,30,16);
  color myForegroundColor = color(#B0FFD0);
  
  float r0 = (float) red(myBackgroundColor);
  float g0 = (float) green(myBackgroundColor);
  float b0 = (float) blue(myBackgroundColor);

  // Fetch the individual color components of myForegroundColor
  float r1 = (float) red(myForegroundColor);
  float g1 = (float) green(myForegroundColor);
  float b1 = (float) blue(myForegroundColor);

  // Fill the red, green and blue arrays with values that
  // blend from myBackgroundColor to myForegroundColor.
  
  for (int i=0; i<256; i++) { 
    
    float f = 0.10 + (float) pow(i/255.0, 0.90);
    if (i==0){
      f = 0.0;
    }

    // Use non-linear color scaling to get a more interesting palette
    float percent  = (float) (i/255.0);
    float rPercent = (float) Math.pow(percent, 0.50) + f;
    float gPercent = (float) Math.pow(percent, 0.40) + f;
    float bPercent = (float) Math.pow(percent, 0.38) + f;
    rPercent = min(1, rPercent); 
    gPercent = min(1, gPercent); 
    bPercent = min(1, bPercent); 

    // Create the actual color component values
    int r   = (int)(r0 + rPercent*(r1-r0));
    int g   = (int)(g0 + gPercent*(g1-g0));
    int b   = (int)(b0 + bPercent*(b1-b0));

    myPalette[i] = color(r,g,b); 
  }
}



//--------------------------------------------------------------------------
// It's good to include some functions which can quickly restore our applet
// to its original (startup) condition.

void clearRaster(){
  for (int i=0; i<numberOfRasterPixels; i++){ 
    rasterPixels[i] = (byte)0;
  }
}
void resetParticles(){
  for (int i=0; i<numberOfParticles; i++){
    particleArray[i].reset();
    particleArray[i].reset2();
  }
}

void keyPressed(){
   clearRaster();
   resetParticles();
   clicked = false;
}


//--------------------------------------------------------------------------
void initializeSimulation(){
  // Initialize the objects and properties of the DLA simulation,
  // namely, a very large array of (invisible) particles.

  // I determined empirically that the applet works well when
  // there are approximately 0.1 Particles per pixel. Larger applets
  // will have simulations which are correspondingly slower to compute.
  final float particlesPerPixel = 0.1f;
  numberOfParticles = (int)(numberOfRasterPixels * particlesPerPixel); 

  // Construct the array containing the Particles.
  particleArray = new Particle[numberOfParticles];
  for (int i=0; i<numberOfParticles; i++){
    particleArray[i] = new Particle();
  }
}

//--------------------------------------------------------------------------
private void computeSimulation(){
  // diffusion-limited aggregation:
  // have a whole bunch of particles which start some distance away
  // each does a drunk walk. if it encounters a part of the structure,
  // it sticks to it, adding on to the structure.

  // slow the growth over time by raising the aggregation threshold.
  aggregationThreshold+=0.11f;

  Particle P;
  int sum;
  int x, y, loc;
  int locc, locn, locs;

  final int minX = 1;
  final int minY = 1;
  final int maxX = rasterWidth-2;
  final int maxY = rasterHeight-2;

  int count = 0;
  for (int i=0; i<numberOfParticles; i++){
    (P = particleArray[i]).update();

    x = (int) P.px; 
    y = (int) P.py;

    if ((x > minX) && (x < maxX) &&
      (y > minY) && (y < maxY)){

      locn = (locc = (loc = y*rasterWidth + x) - 1) - rasterWidth;
      locs = locc + rasterWidth;

      sum =		
        (UNSIGN & rasterPixels [locn++]) +
        (UNSIGN & rasterPixels [locn++]) + 
        (UNSIGN & rasterPixels [locn  ]) + 
        (UNSIGN & rasterPixels [locc++]) + 
        (UNSIGN & rasterPixels [locc++]) + 
        (UNSIGN & rasterPixels [locc  ]) +
        (UNSIGN & rasterPixels [locs++]) + 
        (UNSIGN & rasterPixels [locs++]) +
        (UNSIGN & rasterPixels [locs  ]);

      if (sum >= aggregationThreshold){
        rasterPixels[loc] = (byte) Math.min(255, (UNSIGN & rasterPixels[loc])+ACCRETE_BYTE);
      
        
        if ((count++)%blurFrequency == 0){ 
          if (random(1) < 0.5){
            blurNeighborhood(loc+rasterWidth);
          } else {
            blurNeighborhood(loc+1);
          }
          blurNeighborhood(loc);
      
        } else {
          blur2(loc);
        }

        P.reset();

      }
    }
  }

  if (((float)count/(float)numberOfParticles) > 0.936f){ 
    clearRaster();
    resetParticles();
  }
}




//--------------------------------------------------------------------------
void blur2 (int locC){
  /* 	This method reassigns the brightness of a pixel to be 
   			a weighted average of itself with the pixels in the 3x3
   			neighborhood around it. If its neighbors are dark, it
   			will become darker; if its neighbors are lighter, it 
   			will become lighter.
   		*/

  // First, we'll do a check to make sure that we can blur this pixel.
  // We'll hit problems if the pixel's upper or lower neighbors lie
  // outside the bounds of the rasterPixels array.
  if ((locC >= (rasterWidth + 1)) &&
    (locC < (numberOfRasterPixels - rasterWidth - 1))){

    // We compute the array indices of the neighbors in our 3x3  
    // region, numbering them from 1 through 9 as follows:
    // 1  2  3
    // 4  5  6
    // 7  8  9
    int loc1 = locC - rasterWidth - 1;		// North West
    int loc2 = locC - rasterWidth;	        // North
    int loc3 = locC - rasterWidth + 1;		// North East
    int loc4 = locC - 1;			// West
    int loc5 = locC;				// Center
    int loc6 = locC + 1;			// East
    int loc7 = locC + rasterWidth - 1;		// South West
    int loc8 = locC + rasterWidth;		// South
    int loc9 = locC + rasterWidth + 1;		// South East


    // Now we fetch the values of the nine pixels out of the 
    // rasterPixels byte array. We need to do some unaesthetic
    // bitwise math with the UNSIGN constant in order to extract 
    // the bytes as unsigned integers (from 0...255). Otherwise Java
    // will (by default) cast bytes to signed integers (-128...127).
    int val1 = UNSIGN & rasterPixels[loc1];
    int val2 = UNSIGN & rasterPixels[loc2];
    int val3 = UNSIGN & rasterPixels[loc3];
    int val4 = UNSIGN & rasterPixels[loc4];
    int val5 = UNSIGN & rasterPixels[loc5];
    int val6 = UNSIGN & rasterPixels[loc6];
    int val7 = UNSIGN & rasterPixels[loc7];
    int val8 = UNSIGN & rasterPixels[loc8];
    int val9 = UNSIGN & rasterPixels[loc9];

    // Now we use a 3x3 Gaussian blurring convolution kernel
    // to determine the weighted average of the pixels. This kernel has
    // the following multiplicative weights assigned to each location:
    // 1  2  1
    // 2  4  2
    // 1  2  1

    int sum = (	1*val1 + 2*val2 + 1*val3 + 
      2*val4 + 4*val5 + 2*val6 + 
      1*val7 + 2*val8 + 1*val9 );

    int weightedAverage = sum/16;
    rasterPixels[locC] = (byte)(weightedAverage);
  }
}





//--------------------------------------------------------------------------
void blurNeighborhood (int locC){
  /* 	This method blurs the 3x3 neighborhood of pixels centered  
   			around the pixel whose index in the rasterArray is locC.
   			In order to improve speed, this method was written 
   			without the security checks which would ensure that
   			the locC index -- and the indices of its neighbors -- 
   			were within the bounds of the rasterArray. Calling this 
   			method on a location which is too close to the edge of the 
   			rasterArray may cause an ArrayIndexOutOfBoundsException.
   			
   			We blur the pixels of the 3x3 region in a certain order 
   			-- from the center towards the edges -- in order to minimize
   			any asymmetric effects.
   		*/

  // Define the indices of the North (upper) 
  // and South (lower) neighboring pixels.
  int locN = locC - rasterWidth;
  int locS = locC + rasterWidth;

  // Finally, blur the corner pixels: NW, NE, SW, SE. 
  blur2(locN-1);
  blur2(locN+1);
  blur2(locS-1);
  blur2(locS+1);	

  // Now blur the north, south, west and east pixels				
  blur2(locN  );
  blur2(locS  );
  blur2(locC-1);
  blur2(locC+1);

  // Blur the center pixel first
  blur2(locC);
}

//--------------------------------------------------------------------------
void createLine (int x0, int y0, int x1, int y1){
  // This method takes the x- and y-coordinates of two points,
  // and draws a line between them (in the raster of pixels)
  // by depositing the DRAW_BYTE value into the rasterPixels array.
  // This is a simple method, but not very efficient at all; for a
  // faster technique, please investigate Bresenham's method.

x0 = constrain(x0, 0, rasterWidth-1); 
    y0 = constrain(y0, 0, rasterHeight-1); 
    x1 = constrain(x1, 0, rasterWidth-1); 
    y1 = constrain(y1, 0, rasterHeight-1); 
    
  float dx = x1-x0;
  float dy = y1-y0;
  float dh = (float)(Math.sqrt(dx*dx + dy*dy));

  int x, y;
  int ym, xm, index;
  float percent;

  for (int i=0; i<dh; i++){
    percent = (float)i/dh;
    x = (int)(x0 + percent*dx);
    y = (int)(y0 + percent*dy);
    
    
    ym = y%rasterHeight;
    xm = x%rasterWidth;
    index = ym*rasterWidth +xm;
    rasterPixels[index] = (byte) ( ((UNSIGN & rasterPixels[index]) + DRAW_BYTE));
  }

}


